@@ -19,6 +19,7 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "sort.h"
+#include "charset.h"
 
 
 #ifdef HAVE_PGP
@@ -216,6 +217,8 @@ int mutt_user_is_recipient (HEADER *h)
  * %T = $to_chars
  * %u = user (login) name of author
  * %v = first name of author, unless from self
+ * %y = `x-label:' field (if present)
+ * %Y = `x-label:' field (if present, tree unfolded, and != parent's x-label)
  * %Z = status flags	*/
 
 struct hdr_format_info
@@ -224,6 +227,36 @@ struct hdr_format_info
   HEADER *hdr;
 };
 
+static void hdr_format_s (char *dest,
+			  size_t destlen,
+			  const char *prefix,
+			  char *s)
+{
+  int right_justify = 0;
+  char *p;
+  int min_width;
+  int max_width = INT_MAX;
+
+  if (*prefix == '-')
+    ++prefix, right_justify = 0;
+  min_width = strtol (prefix, &p, 10);
+  if (p > prefix)
+  {
+    if (*p == '.')
+    {
+      prefix = p + 1;
+      max_width = strtol (prefix, &p, 10);
+      if (p <= prefix)
+	max_width = INT_MAX;
+    }
+  }
+  else
+    min_width = 0;
+
+  mutt_format_string (dest, destlen, min_width, max_width,
+		      right_justify, ' ', s, -1);
+}
+
 static const char *
 hdr_format_str (char *dest,
 		size_t destlen,
@@ -236,7 +269,7 @@ hdr_format_str (char *dest,
 		format_flag flags)
 {
   struct hdr_format_info *hfi = (struct hdr_format_info *) data;
-  HEADER *hdr;
+  HEADER *hdr, *htmp;
   CONTEXT *ctx;
   char fmt[SHORT_STRING], buf2[SHORT_STRING], ch, *p;
   int do_locales, i;
@@ -256,8 +289,7 @@ hdr_format_str (char *dest,
     case 'a':
       if(hdr->env->from && hdr->env->from->mailbox)
       {
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, hdr->env->from->mailbox);
+	hdr_format_s (dest, destlen, prefix, hdr->env->from->mailbox);
       }
       else
         dest[0] = '\0';
@@ -270,8 +302,7 @@ hdr_format_str (char *dest,
       if (dest[0])
       {
 	strfcpy (buf2, dest, sizeof(buf2));
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
+	snprintf (dest, destlen, prefix, buf2);
 	break;
       }
       /* fall through if 'B' returns nothing */
@@ -286,15 +317,13 @@ hdr_format_str (char *dest,
       }
       else 
 	strfcpy(dest, "(null)", destlen);
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
       strfcpy (buf2, dest, sizeof(buf2));
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
     
     case 'c':
       mutt_pretty_size (buf2, sizeof (buf2), (long) hdr->content->length);
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'C':
@@ -397,34 +426,40 @@ hdr_format_str (char *dest,
 	if (do_locales)
 	  setlocale (LC_TIME, "C");
 
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
+	hdr_format_s (dest, destlen, prefix, buf2);
 	if (len > 0 && op != 'd')
 	  src = cp + 1;
       }
       break;
 
+    case 'e':
+      snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+      snprintf (dest, destlen, fmt, mutt_msgno_in_thread(hdr) + 1);
+      break;
+
+    case 'E':
+      snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+      snprintf (dest, destlen, fmt, mutt_messages_in_thread(hdr));
+      break;
+
     case 'f':
       buf2[0] = 0;
       rfc822_write_address (buf2, sizeof (buf2), hdr->env->from);
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'F':
       if (!optional)
       {
-        snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
         make_from (hdr->env, buf2, sizeof (buf2), 0);
-        snprintf (dest, destlen, fmt, buf2);
+	hdr_format_s (dest, destlen, prefix, buf2);
       }
       else if (mutt_addr_is_user (hdr->env->from))
         optional = 0;
       break;
 
     case 'i':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, hdr->env->message_id ? hdr->env->message_id : "<no.id>");
+      hdr_format_s (dest, destlen, prefix, hdr->env->message_id ? hdr->env->message_id : "<no.id>");
       break;
 
     case 'l':
@@ -444,8 +479,7 @@ hdr_format_str (char *dest,
       if (!optional)
       {
 	make_from (hdr->env, buf2, sizeof (buf2), 1);
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
+	hdr_format_s (dest, destlen, prefix, buf2);
       }
       else if (!check_for_mailing_list (hdr->env->to, NULL, NULL, 0) &&
 	       !check_for_mailing_list (hdr->env->cc, NULL, NULL, 0))
@@ -465,8 +499,7 @@ hdr_format_str (char *dest,
       break;
 
     case 'n':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, mutt_get_name (hdr->env->from));
+      hdr_format_s (dest, destlen, prefix, mutt_get_name (hdr->env->from));
       break;
 
     case 'N':
@@ -488,8 +521,7 @@ hdr_format_str (char *dest,
 	make_from_addr (hdr->env, buf2, sizeof (buf2), 1);
 	if (!option (OPTSAVEADDRESS) && (p = strpbrk (buf2, "%@")))
 	  *p = 0;
-	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-	snprintf (dest, destlen, fmt, buf2);
+	hdr_format_s (dest, destlen, prefix, buf2);
       }
       else if (!check_for_mailing_list_addr (hdr->env->to, NULL, 0) &&
 	       !check_for_mailing_list_addr (hdr->env->cc, NULL, 0))
@@ -500,13 +532,12 @@ hdr_format_str (char *dest,
 
     case 'M':
       snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
-      snprintf (buf2, sizeof (buf2), "%%%ss", prefix);
       if (!optional)
       {
 	if (threads && is_index && hdr->collapsed && hdr->num_hidden > 1)
 	  snprintf (dest, destlen, fmt, hdr->num_hidden);
 	else if (is_index && threads)
-	  snprintf (dest, destlen, buf2, " ");
+	  hdr_format_s (dest, destlen, prefix, " ");
 	else
 	  *dest = '\0';
       }
@@ -519,19 +550,18 @@ hdr_format_str (char *dest,
 
     case 's':
       
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
       if (flags & M_FORMAT_TREE)
       {
 	if (flags & M_FORMAT_FORCESUBJ)
 	{
 	  snprintf (buf2, sizeof (buf2), "%s%s", hdr->tree, NONULL (hdr->env->subject));
-	  snprintf (dest, destlen, fmt, buf2);
+	  hdr_format_s (dest, destlen, prefix, buf2);
 	}
 	else
-	  snprintf (dest, destlen, fmt, hdr->tree);
+	  hdr_format_s (dest, destlen, prefix, hdr->tree);
       }
       else
-	snprintf (dest, destlen, fmt, NONULL (hdr->env->subject));
+	hdr_format_s (dest, destlen, prefix, NONULL (hdr->env->subject));
       break;
 
     case 'S':
@@ -568,8 +598,7 @@ hdr_format_str (char *dest,
 	else if (hdr->env->cc)
 	  snprintf (buf2, sizeof (buf2), "Cc %s", mutt_get_name (hdr->env->cc));
       }
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'T':
@@ -587,26 +616,24 @@ hdr_format_str (char *dest,
       }
       else
 	buf2[0] = 0;
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'v':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
       if (mutt_addr_is_user (hdr->env->from)) 
       {
 	if (hdr->env->to)
-	  snprintf (buf2, sizeof (buf2), fmt, mutt_get_name (hdr->env->to));
+	  hdr_format_s (buf2, sizeof (buf2), prefix, mutt_get_name (hdr->env->to));
 	else if (hdr->env->cc)
-	  snprintf (buf2, sizeof (buf2), fmt, mutt_get_name (hdr->env->cc));
+	  hdr_format_s (buf2, sizeof (buf2), prefix, mutt_get_name (hdr->env->cc));
 	else
 	  *buf2 = 0;
       }
       else
-	snprintf (buf2, sizeof (buf2), fmt, mutt_get_name (hdr->env->from));
+	hdr_format_s (buf2, sizeof (buf2), prefix, mutt_get_name (hdr->env->from));
       if ((p = strpbrk (buf2, " %@")))
 	*p = 0;
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
 
     case 'Z':
@@ -624,7 +651,6 @@ hdr_format_str (char *dest,
         ch = 'K';
 #endif
 
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
       snprintf (buf2, sizeof (buf2),
 		"%c%c%c", (THREAD_NEW ? 'n' : (THREAD_OLD ? 'o' : 
 		((hdr->read && (ctx && ctx->msgnotreadyet != hdr->msgno))
@@ -633,9 +659,44 @@ hdr_format_str (char *dest,
 		hdr->tagged ? '*' :
 		(hdr->flagged ? '!' :
 		 (Tochars && ((i = mutt_user_is_recipient (hdr)) < mutt_strlen (Tochars)) ? Tochars[i] : ' ')));
-      snprintf (dest, destlen, fmt, buf2);
+      hdr_format_s (dest, destlen, prefix, buf2);
       break;
 
+     case 'y':
+       if (optional)
+	 optional = hdr->env->x_label ? 1 : 0;
+
+       hdr_format_s (dest, destlen, prefix, NONULL (hdr->env->x_label));
+       break;
+ 
+    case 'Y':
+      if (hdr->env->x_label)
+      {
+	i = 1;	/* reduce reuse recycle */
+	htmp = NULL;
+	if (flags & M_FORMAT_TREE
+	    && (hdr->prev && hdr->prev->env->x_label))
+	  htmp = hdr->prev;
+	else if (flags & M_FORMAT_TREE
+		 && (hdr->parent && hdr->parent->env->x_label))
+	  htmp = hdr->parent;
+	if (htmp && mutt_strcasecmp (hdr->env->x_label,
+				     htmp->env->x_label) == 0)
+	  i = 0;
+      }
+      else
+	i = 0;
+
+      if (optional)
+	optional = i;
+
+      if (i)
+        hdr_format_s (dest, destlen, prefix, NONULL (hdr->env->x_label));
+      else
+        hdr_format_s (dest, destlen, prefix, "");
+
+      break;
+      
     default:
       snprintf (dest, destlen, "%%%s%c", prefix, op);
       break;
