@@ -783,6 +783,8 @@ struct option_t MuttVars[] = {
   ** %D      date and time of the message in the format
   ** .       specified by ``date_format'' converted to 
   ** .       the local time zone
+  ** %e      current message number in thread
+  ** %E      number of messages in current thread
   ** %f      entire From: line (address + real name)
   ** %F      author name, or recipient name if the 
   ** .       message is from you
@@ -806,6 +808,12 @@ struct option_t MuttVars[] = {
   ** %u      user (login) name of the author
   ** %v      first name of the author, or the 
   ** .       recipient if the message is from you
+  ** %y	     `x-label:' field, if present
+  ** %Y	     `x-label' field, if present, and 
+  ** .       (1) not at part of a thread tree,
+  ** .       (2) at the top of a thread, or
+  ** . 	     (3) `x-label' is different from preceding
+  ** .       message's `x-label'.
   ** %Z      message status flags
   ** %{fmt}  the date and time of the message is
   ** .       converted to sender's time zone, and 
@@ -1154,7 +1162,7 @@ struct option_t MuttVars[] = {
   ** .pp
   ** If you have more than one key pair, this option allows you to specify
   ** which of your private keys to use.  It is recommended that you use the
-  ** keyid form to specify your key (e.g., ``0x00112233'').
+  ** keyid form to specify your key (e.g., ``0xABCDEFGH'').
   */
   { "pgp_sign_micalg",	DT_STR,	 R_NONE, UL &PgpSignMicalg, UL "pgp-md5" },
   /*
@@ -1347,19 +1355,6 @@ struct option_t MuttVars[] = {
   ** if you accept it or not. If you accept it, the certificate can also 
   ** be saved in this file and further connections are automatically 
   ** accepted.
-  ** .pp
-  ** You can also manually add CA certificates in this file. Any server
-  ** certificate that is signed with one of these CA certificates are 
-  ** also automatically accepted.
-  ** .pp
-  ** Example: set certificate_file=~/.mutt/certificates
-  */
-  { "ssl_usesystemcerts", DT_BOOL, R_NONE, OPTSSLSYSTEMCERTS, 1 },
-  /*
-  ** .pp
-  ** If set to \fIyes\fP, mutt will use CA certificates in the
-  ** system-wide certificate store when checking if server certificate 
-  ** is signed by a trusted CA.
   */
   { "entropy_file",	DT_PATH, R_NONE, UL &SslEntropyFile, 0 },
   /* .pp
@@ -1411,7 +1406,7 @@ struct option_t MuttVars[] = {
   ** messages to an external Unix command.
   */
 #ifdef USE_POP
-  { "pop_delete",	DT_BOOL, R_NONE, OPTPOPDELETE, 0 },
+  { "pop_delete",	DT_QUAD, R_NONE, OPTPOPDELETE, M_ASKNO },
   /*
   ** .pp
   ** If set, Mutt will delete successfully downloaded messages from the POP
