@@ -45,7 +45,8 @@
 
 # ifdef ENABLE_NLS
 #  include <libintl.h>
-#  define _(a) (gettext (a))
+char *mutt_gettext (const char *); /* gettext.c */
+# define _(a) (mutt_gettext (a))
 #  ifdef gettext_noop
 #   define N_(a) gettext_noop (a)
 #  else
